package zoo.wbooster

import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.net.Uri
import android.os.Build
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.os.IBinder
import android.view.View
import androidx.annotation.RequiresApi
import androidx.core.content.ContextCompat
import kotlinx.android.synthetic.main.activity_fullscreen.*
import java.net.URI


/**
 * Full screen activity devoted to play music
 */
class FullscreenActivity : AppCompatActivity() {

    private lateinit var mediaPlayerService: MediaPlayerService
    private lateinit var downloaderService: DownloaderService
    private var isDownloaderServiceBound: Boolean = false
    private var isMediaPlayerServiceBound: Boolean = false


    private val downloaderConnection = object : ServiceConnection {
        override fun onServiceConnected(name: ComponentName?, service: IBinder?) {
            val binder = service as DownloaderService.DownloaderBinder
            downloaderService = binder.getService()
            isDownloaderServiceBound = true
        }

        override fun onServiceDisconnected(name: ComponentName?) {
            isDownloaderServiceBound = false
        }
    }

    private val mediaPlayerConnection = object : ServiceConnection {
        override fun onServiceConnected(name: ComponentName?, service: IBinder?) {
            val binder = service as MediaPlayerService.MediaPlayerBinder
            mediaPlayerService = binder.getService()
            isMediaPlayerServiceBound = true
        }

        override fun onServiceDisconnected(name: ComponentName?) {
            isMediaPlayerServiceBound = false
        }
    }

    override fun onStart() {
        super.onStart()
        ContextCompat.startForegroundService(this, Intent(this, MediaPlayerService::class.java))
        Intent(this, MediaPlayerService::class.java).also {
            bindService(it, mediaPlayerConnection, Context.BIND_AUTO_CREATE)
        }
        Intent(this, DownloaderService::class.java).also {
            bindService(it, downloaderConnection, Context.BIND_AUTO_CREATE)
        }

    }

    override fun onStop() {
        super.onStop()
        unbindService(mediaPlayerConnection)
        unbindService(downloaderConnection)
        isMediaPlayerServiceBound = false
        isDownloaderServiceBound = false
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        // Upgrade Youtube-dl
//        YT.updateYoutubeDL(application)

        // Set fullscreen immersive mode
        setContentView(R.layout.activity_fullscreen)
        fullscreen_content.systemUiVisibility =
            View.SYSTEM_UI_FLAG_LOW_PROFILE or
                    View.SYSTEM_UI_FLAG_FULLSCREEN or
                    View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY or
                    View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION or
                    View.SYSTEM_UI_FLAG_HIDE_NAVIGATION or
                    View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN


        fullscreen_content_controls.visibility = View.VISIBLE
    }

    @RequiresApi(Build.VERSION_CODES.O)
    fun startPlaylist(v: View){
        playButton.setOnClickListener{} // Remove listner
        playButton.isEnabled = false

        if (!isMediaPlayerServiceBound || !isDownloaderServiceBound){
            println("Problem connecting to services")
            println("Media player status: $isMediaPlayerServiceBound")
            println("Downloader status: $isDownloaderServiceBound")
            return
        }

        // Setup title update lambda
        mediaPlayerService.titleUpdater = {
            runOnUiThread(Runnable {
                currently_playing.text = " $it    ".repeat(400)
                currently_playing.isSelected = true
            })
        }
        // Setup image update lambda
        mediaPlayerService.thumbnailUpdater = {
            runOnUiThread(Runnable {
                thumbnail_view.setImageURI(Uri.parse(it))
            })
        }


        downloaderService.startDownloading(mediaPlayerService, downloadBar)
        mediaPlayerService.startPlaying()
    }
}
